@@ -30,107 +30,107 @@ namespace nghttp2 {
 
 namespace http2 {
 
-StringRef get_status_string(BlockAllocator &balloc, unsigned int status_code) {
+StringRef get_reason_phrase(unsigned int status_code) {
   switch (status_code) {
   case 100:
-    return StringRef::from_lit("100 Continue");
+    return StringRef::from_lit("Continue");
   case 101:
-    return StringRef::from_lit("101 Switching Protocols");
+    return StringRef::from_lit("Switching Protocols");
   case 200:
-    return StringRef::from_lit("200 OK");
+    return StringRef::from_lit("OK");
   case 201:
-    return StringRef::from_lit("201 Created");
+    return StringRef::from_lit("Created");
   case 202:
-    return StringRef::from_lit("202 Accepted");
+    return StringRef::from_lit("Accepted");
   case 203:
-    return StringRef::from_lit("203 Non-Authoritative Information");
+    return StringRef::from_lit("Non-Authoritative Information");
   case 204:
-    return StringRef::from_lit("204 No Content");
+    return StringRef::from_lit("No Content");
   case 205:
-    return StringRef::from_lit("205 Reset Content");
+    return StringRef::from_lit("Reset Content");
   case 206:
-    return StringRef::from_lit("206 Partial Content");
+    return StringRef::from_lit("Partial Content");
   case 300:
-    return StringRef::from_lit("300 Multiple Choices");
+    return StringRef::from_lit("Multiple Choices");
   case 301:
-    return StringRef::from_lit("301 Moved Permanently");
+    return StringRef::from_lit("Moved Permanently");
   case 302:
-    return StringRef::from_lit("302 Found");
+    return StringRef::from_lit("Found");
   case 303:
-    return StringRef::from_lit("303 See Other");
+    return StringRef::from_lit("See Other");
   case 304:
-    return StringRef::from_lit("304 Not Modified");
+    return StringRef::from_lit("Not Modified");
   case 305:
-    return StringRef::from_lit("305 Use Proxy");
-  // case 306: return StringRef::from_lit("306 (Unused)");
+    return StringRef::from_lit("Use Proxy");
+  // case 306: return StringRef::from_lit("(Unused)");
   case 307:
-    return StringRef::from_lit("307 Temporary Redirect");
+    return StringRef::from_lit("Temporary Redirect");
   case 308:
-    return StringRef::from_lit("308 Permanent Redirect");
+    return StringRef::from_lit("Permanent Redirect");
   case 400:
-    return StringRef::from_lit("400 Bad Request");
+    return StringRef::from_lit("Bad Request");
   case 401:
-    return StringRef::from_lit("401 Unauthorized");
+    return StringRef::from_lit("Unauthorized");
   case 402:
-    return StringRef::from_lit("402 Payment Required");
+    return StringRef::from_lit("Payment Required");
   case 403:
-    return StringRef::from_lit("403 Forbidden");
+    return StringRef::from_lit("Forbidden");
   case 404:
-    return StringRef::from_lit("404 Not Found");
+    return StringRef::from_lit("Not Found");
   case 405:
-    return StringRef::from_lit("405 Method Not Allowed");
+    return StringRef::from_lit("Method Not Allowed");
   case 406:
-    return StringRef::from_lit("406 Not Acceptable");
+    return StringRef::from_lit("Not Acceptable");
   case 407:
-    return StringRef::from_lit("407 Proxy Authentication Required");
+    return StringRef::from_lit("Proxy Authentication Required");
   case 408:
-    return StringRef::from_lit("408 Request Timeout");
+    return StringRef::from_lit("Request Timeout");
   case 409:
-    return StringRef::from_lit("409 Conflict");
+    return StringRef::from_lit("Conflict");
   case 410:
-    return StringRef::from_lit("410 Gone");
+    return StringRef::from_lit("Gone");
   case 411:
-    return StringRef::from_lit("411 Length Required");
+    return StringRef::from_lit("Length Required");
   case 412:
-    return StringRef::from_lit("412 Precondition Failed");
+    return StringRef::from_lit("Precondition Failed");
   case 413:
-    return StringRef::from_lit("413 Payload Too Large");
+    return StringRef::from_lit("Payload Too Large");
   case 414:
-    return StringRef::from_lit("414 URI Too Long");
+    return StringRef::from_lit("URI Too Long");
   case 415:
-    return StringRef::from_lit("415 Unsupported Media Type");
+    return StringRef::from_lit("Unsupported Media Type");
   case 416:
-    return StringRef::from_lit("416 Requested Range Not Satisfiable");
+    return StringRef::from_lit("Requested Range Not Satisfiable");
   case 417:
-    return StringRef::from_lit("417 Expectation Failed");
+    return StringRef::from_lit("Expectation Failed");
   case 421:
-    return StringRef::from_lit("421 Misdirected Request");
+    return StringRef::from_lit("Misdirected Request");
   case 426:
-    return StringRef::from_lit("426 Upgrade Required");
+    return StringRef::from_lit("Upgrade Required");
   case 428:
-    return StringRef::from_lit("428 Precondition Required");
+    return StringRef::from_lit("Precondition Required");
   case 429:
-    return StringRef::from_lit("429 Too Many Requests");
+    return StringRef::from_lit("Too Many Requests");
   case 431:
-    return StringRef::from_lit("431 Request Header Fields Too Large");
+    return StringRef::from_lit("Request Header Fields Too Large");
   case 451:
-    return StringRef::from_lit("451 Unavailable For Legal Reasons");
+    return StringRef::from_lit("Unavailable For Legal Reasons");
   case 500:
-    return StringRef::from_lit("500 Internal Server Error");
+    return StringRef::from_lit("Internal Server Error");
   case 501:
-    return StringRef::from_lit("501 Not Implemented");
+    return StringRef::from_lit("Not Implemented");
   case 502:
-    return StringRef::from_lit("502 Bad Gateway");
+    return StringRef::from_lit("Bad Gateway");
   case 503:
-    return StringRef::from_lit("503 Service Unavailable");
+    return StringRef::from_lit("Service Unavailable");
   case 504:
-    return StringRef::from_lit("504 Gateway Timeout");
+    return StringRef::from_lit("Gateway Timeout");
   case 505:
-    return StringRef::from_lit("505 HTTP Version Not Supported");
+    return StringRef::from_lit("HTTP Version Not Supported");
   case 511:
-    return StringRef::from_lit("511 Network Authentication Required");
+    return StringRef::from_lit("Network Authentication Required");
   default:
-    return util::make_string_ref_uint(balloc, status_code);
+    return StringRef{};
   }
 }
 
@@ -358,15 +358,21 @@ nghttp2_nv make_nv_nocopy(const StringRef &name, const StringRef &value,
 
 namespace {
 void copy_headers_to_nva_internal(std::vector<nghttp2_nv> &nva,
-                                  const HeaderRefs &headers, uint8_t nv_flags) {
-  for (auto &kv : headers) {
-    if (kv.name.empty() || kv.name[0] == ':') {
+                                  const HeaderRefs &headers, uint8_t nv_flags,
+                                  uint32_t flags) {
+  auto it_forwarded = std::end(headers);
+  auto it_xff = std::end(headers);
+  auto it_xfp = std::end(headers);
+  auto it_via = std::end(headers);
+
+  for (auto it = std::begin(headers); it != std::end(headers); ++it) {
+    auto kv = &(*it);
+    if (kv->name.empty() || kv->name[0] == ':') {
       continue;
     }
-    switch (kv.token) {
+    switch (kv->token) {
     case HD_COOKIE:
     case HD_CONNECTION:
-    case HD_FORWARDED:
     case HD_HOST:
     case HD_HTTP2_SETTINGS:
     case HD_KEEP_ALIVE:
@@ -375,51 +381,157 @@ void copy_headers_to_nva_internal(std::vector<nghttp2_nv> &nva,
     case HD_TE:
     case HD_TRANSFER_ENCODING:
     case HD_UPGRADE:
-    case HD_VIA:
-    case HD_X_FORWARDED_FOR:
-    case HD_X_FORWARDED_PROTO:
       continue;
+    case HD_FORWARDED:
+      if (flags & HDOP_STRIP_FORWARDED) {
+        continue;
+      }
+
+      if (it_forwarded == std::end(headers)) {
+        it_forwarded = it;
+        continue;
+      }
+
+      kv = &(*it_forwarded);
+      it_forwarded = it;
+      break;
+    case HD_X_FORWARDED_FOR:
+      if (flags & HDOP_STRIP_X_FORWARDED_FOR) {
+        continue;
+      }
+
+      if (it_xff == std::end(headers)) {
+        it_xff = it;
+        continue;
+      }
+
+      kv = &(*it_xff);
+      it_xff = it;
+      break;
+    case HD_X_FORWARDED_PROTO:
+      if (flags & HDOP_STRIP_X_FORWARDED_PROTO) {
+        continue;
+      }
+
+      if (it_xfp == std::end(headers)) {
+        it_xfp = it;
+        continue;
+      }
+
+      kv = &(*it_xfp);
+      it_xfp = it;
+      break;
+    case HD_VIA:
+      if (flags & HDOP_STRIP_VIA) {
+        continue;
+      }
+
+      if (it_via == std::end(headers)) {
+        it_via = it;
+        continue;
+      }
+
+      kv = &(*it_via);
+      it_via = it;
+      break;
     }
-    nva.push_back(make_nv_internal(kv.name, kv.value, kv.no_index, nv_flags));
+    nva.push_back(
+        make_nv_internal(kv->name, kv->value, kv->no_index, nv_flags));
   }
 }
 } // namespace
 
 void copy_headers_to_nva(std::vector<nghttp2_nv> &nva,
-                         const HeaderRefs &headers) {
-  copy_headers_to_nva_internal(nva, headers, NGHTTP2_NV_FLAG_NONE);
+                         const HeaderRefs &headers, uint32_t flags) {
+  copy_headers_to_nva_internal(nva, headers, NGHTTP2_NV_FLAG_NONE, flags);
 }
 
 void copy_headers_to_nva_nocopy(std::vector<nghttp2_nv> &nva,
-                                const HeaderRefs &headers) {
-  copy_headers_to_nva_internal(nva, headers, NGHTTP2_NV_FLAG_NO_COPY_NAME |
-                                                 NGHTTP2_NV_FLAG_NO_COPY_VALUE);
+                                const HeaderRefs &headers, uint32_t flags) {
+  copy_headers_to_nva_internal(
+      nva, headers,
+      NGHTTP2_NV_FLAG_NO_COPY_NAME | NGHTTP2_NV_FLAG_NO_COPY_VALUE, flags);
 }
 
 void build_http1_headers_from_headers(DefaultMemchunks *buf,
-                                      const HeaderRefs &headers) {
-  for (auto &kv : headers) {
-    if (kv.name.empty() || kv.name[0] == ':') {
+                                      const HeaderRefs &headers,
+                                      uint32_t flags) {
+  auto it_forwarded = std::end(headers);
+  auto it_xff = std::end(headers);
+  auto it_xfp = std::end(headers);
+  auto it_via = std::end(headers);
+
+  for (auto it = std::begin(headers); it != std::end(headers); ++it) {
+    auto kv = &(*it);
+    if (kv->name.empty() || kv->name[0] == ':') {
       continue;
     }
-    switch (kv.token) {
+    switch (kv->token) {
     case HD_CONNECTION:
     case HD_COOKIE:
-    case HD_FORWARDED:
     case HD_HOST:
     case HD_HTTP2_SETTINGS:
     case HD_KEEP_ALIVE:
     case HD_PROXY_CONNECTION:
     case HD_SERVER:
     case HD_UPGRADE:
-    case HD_VIA:
-    case HD_X_FORWARDED_FOR:
-    case HD_X_FORWARDED_PROTO:
       continue;
+    case HD_FORWARDED:
+      if (flags & HDOP_STRIP_FORWARDED) {
+        continue;
+      }
+
+      if (it_forwarded == std::end(headers)) {
+        it_forwarded = it;
+        continue;
+      }
+
+      kv = &(*it_forwarded);
+      it_forwarded = it;
+      break;
+    case HD_X_FORWARDED_FOR:
+      if (flags & HDOP_STRIP_X_FORWARDED_FOR) {
+        continue;
+      }
+
+      if (it_xff == std::end(headers)) {
+        it_xff = it;
+        continue;
+      }
+
+      kv = &(*it_xff);
+      it_xff = it;
+      break;
+    case HD_X_FORWARDED_PROTO:
+      if (flags & HDOP_STRIP_X_FORWARDED_PROTO) {
+        continue;
+      }
+
+      if (it_xfp == std::end(headers)) {
+        it_xfp = it;
+        continue;
+      }
+
+      kv = &(*it_xfp);
+      it_xfp = it;
+      break;
+    case HD_VIA:
+      if (flags & HDOP_STRIP_VIA) {
+        continue;
+      }
+
+      if (it_via == std::end(headers)) {
+        it_via = it;
+        continue;
+      }
+
+      kv = &(*it_via);
+      it_via = it;
+      break;
     }
-    capitalize(buf, kv.name);
+    capitalize(buf, kv->name);
     buf->append(": ");
-    buf->append(kv.value);
+    buf->append(kv->value);
     buf->append("\r\n");
   }
 }
@@ -477,6 +589,11 @@ void dump_nv(FILE *out, const HeaderRefs &nva) {
   fflush(out);
 }
 
+void erase_header(HeaderRef *hd) {
+  hd->name = StringRef{};
+  hd->token = -1;
+}
+
 StringRef rewrite_location_uri(BlockAllocator &balloc, const StringRef &uri,
                                const http_parser_url &u,
                                const StringRef &match_host,
@@ -1507,9 +1624,9 @@ StringRef path_join(BlockAllocator &balloc, const StringRef &base_path,
                     const StringRef &base_query, const StringRef &rel_path,
                     const StringRef &rel_query) {
   auto res = make_byte_ref(
-      balloc, std::max(static_cast<size_t>(1), base_path.size()) +
-                  rel_path.size() + 1 +
-                  std::max(base_query.size(), rel_query.size()) + 1);
+      balloc,
+      std::max(static_cast<size_t>(1), base_path.size()) + rel_path.size() + 1 +
+          std::max(base_query.size(), rel_query.size()) + 1);
   auto p = res.base;
 
   if (rel_path.empty()) {
@@ -1665,6 +1782,31 @@ StringRef copy_lower(BlockAllocator &balloc, const StringRef &src) {
   return StringRef{iov.base, p};
 }
 
+bool contains_trailers(const StringRef &s) {
+  constexpr auto trailers = StringRef::from_lit("trailers");
+
+  for (auto p = std::begin(s), end = std::end(s);; ++p) {
+    p = std::find_if(p, end, [](char c) { return c != ' ' && c != '\t'; });
+    if (p == end || static_cast<size_t>(end - p) < trailers.size()) {
+      return false;
+    }
+    if (util::strieq(trailers, StringRef{p, p + trailers.size()})) {
+      // Make sure that there is no character other than white spaces
+      // before next "," or end of string.
+      p = std::find_if(p + trailers.size(), end,
+                       [](char c) { return c != ' ' && c != '\t'; });
+      if (p == end || *p == ',') {
+        return true;
+      }
+    }
+    // Skip to next ",".
+    p = std::find_if(p, end, [](char c) { return c == ','; });
+    if (p == end) {
+      return false;
+    }
+  }
+}
+
 } // namespace http2
 
 } // namespace nghttp2
