@@ -131,11 +131,10 @@ bool in_attr_char(char c) {
          std::find(std::begin(bad), std::end(bad), c) == std::end(bad);
 }
 
-std::string percent_encode_token(const std::string &target) {
-  std::string dest;
-
-  dest.resize(target.size() * 3);
-  auto p = std::begin(dest);
+StringRef percent_encode_token(BlockAllocator &balloc,
+                               const StringRef &target) {
+  auto iov = make_byte_ref(balloc, target.size() * 3 + 1);
+  auto p = iov.base;
 
   for (auto first = std::begin(target); first != std::end(target); ++first) {
     uint8_t c = *first;
@@ -149,8 +148,10 @@ std::string percent_encode_token(const std::string &target) {
     *p++ = UPPER_XDIGITS[c >> 4];
     *p++ = UPPER_XDIGITS[(c & 0x0f)];
   }
-  dest.resize(p - std::begin(dest));
-  return dest;
+
+  *p = '\0';
+
+  return StringRef{iov.base, p};
 }
 
 uint32_t hex_to_uint(char c) {
@@ -166,25 +167,27 @@ uint32_t hex_to_uint(char c) {
   return c;
 }
 
-std::string quote_string(const std::string &target) {
+StringRef quote_string(BlockAllocator &balloc, const StringRef &target) {
   auto cnt = std::count(std::begin(target), std::end(target), '"');
 
   if (cnt == 0) {
-    return target;
+    return make_string_ref(balloc, target);
   }
 
-  std::string res;
-  res.reserve(target.size() + cnt);
+  auto iov = make_byte_ref(balloc, target.size() + cnt + 1);
+  auto p = iov.base;
 
   for (auto c : target) {
     if (c == '"') {
-      res += "\\\"";
+      *p++ = '\\';
+      *p++ = '"';
     } else {
-      res += c;
+      *p++ = c;
     }
   }
+  *p = '\0';
 
-  return res;
+  return StringRef{iov.base, p};
 }
 
 namespace {
@@ -202,23 +205,27 @@ Iterator cpydig(Iterator d, uint32_t n, size_t len) {
 } // namespace
 
 namespace {
-const char *MONTH[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun",
-                       "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
-const char *DAY_OF_WEEK[] = {"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};
+constexpr const char *MONTH[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun",
+                                 "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
+constexpr const char *DAY_OF_WEEK[] = {"Sun", "Mon", "Tue", "Wed",
+                                       "Thu", "Fri", "Sat"};
 } // namespace
 
 std::string http_date(time_t t) {
+  /* Sat, 27 Sep 2014 06:31:15 GMT */
+  std::string res(29, 0);
+  http_date(&res[0], t);
+  return res;
+}
+
+char *http_date(char *res, time_t t) {
   struct tm tms;
-  std::string res;
 
   if (gmtime_r(&t, &tms) == nullptr) {
     return res;
   }
 
-  /* Sat, 27 Sep 2014 06:31:15 GMT */
-  res.resize(29);
-
-  auto p = std::begin(res);
+  auto p = res;
 
   auto s = DAY_OF_WEEK[tms.tm_wday];
   p = std::copy_n(s, 3, p);
@@ -239,22 +246,24 @@ std::string http_date(time_t t) {
   s = " GMT";
   p = std::copy_n(s, 4, p);
 
-  return res;
+  return p;
 }
 
 std::string common_log_date(time_t t) {
+  // 03/Jul/2014:00:19:38 +0900
+  std::string res(26, 0);
+  common_log_date(&res[0], t);
+  return res;
+}
+
+char *common_log_date(char *res, time_t t) {
   struct tm tms;
 
   if (localtime_r(&t, &tms) == nullptr) {
-    return "";
+    return res;
   }
 
-  // Format data like this:
-  // 03/Jul/2014:00:19:38 +0900
-  std::string res;
-  res.resize(26);
-
-  auto p = std::begin(res);
+  auto p = res;
 
   p = cpydig(p, tms.tm_mday, 2);
   *p++ = '/';
@@ -285,24 +294,27 @@ std::string common_log_date(time_t t) {
   p = cpydig(p, gmtoff / 3600, 2);
   p = cpydig(p, (gmtoff % 3600) / 60, 2);
 
-  return res;
+  return p;
 }
 
 std::string iso8601_date(int64_t ms) {
+  // 2014-11-15T12:58:24.741Z
+  // 2014-11-15T12:58:24.741+09:00
+  std::string res(29, 0);
+  auto p = iso8601_date(&res[0], ms);
+  res.resize(p - &res[0]);
+  return res;
+}
+
+char *iso8601_date(char *res, int64_t ms) {
   time_t sec = ms / 1000;
 
   tm tms;
   if (localtime_r(&sec, &tms) == nullptr) {
-    return "";
+    return res;
   }
 
-  // Format data like this:
-  // 2014-11-15T12:58:24.741Z
-  // 2014-11-15T12:58:24.741+09:00
-  std::string res;
-  res.resize(29);
-
-  auto p = std::begin(res);
+  auto p = res;
 
   p = cpydig(p, tms.tm_year + 1900, 4);
   *p++ = '-';
@@ -337,9 +349,7 @@ std::string iso8601_date(int64_t ms) {
     p = cpydig(p, (gmtoff % 3600) / 60, 2);
   }
 
-  res.resize(p - std::begin(res));
-
-  return res;
+  return p;
 }
 
 time_t parse_http_date(const StringRef &s) {
@@ -360,7 +370,7 @@ char upcase(char c) {
 }
 
 namespace {
-const char LOWER_XDIGITS[] = "0123456789abcdef";
+constexpr char LOWER_XDIGITS[] = "0123456789abcdef";
 } // namespace
 
 std::string format_hex(const unsigned char *s, size_t len) {
@@ -376,6 +386,21 @@ std::string format_hex(const unsigned char *s, size_t len) {
   return res;
 }
 
+StringRef format_hex(BlockAllocator &balloc, const StringRef &s) {
+  auto iov = make_byte_ref(balloc, s.size() * 2 + 1);
+  auto p = iov.base;
+
+  for (auto cc : s) {
+    uint8_t c = cc;
+    *p++ = LOWER_XDIGITS[c >> 4];
+    *p++ = LOWER_XDIGITS[c & 0xf];
+  }
+
+  *p = '\0';
+
+  return StringRef{iov.base, p};
+}
+
 void to_token68(std::string &base64str) {
   std::transform(std::begin(base64str), std::end(base64str),
                  std::begin(base64str), [](char c) {
@@ -392,22 +417,32 @@ void to_token68(std::string &base64str) {
                   std::end(base64str));
 }
 
-void to_base64(std::string &token68str) {
-  std::transform(std::begin(token68str), std::end(token68str),
-                 std::begin(token68str), [](char c) {
-                   switch (c) {
-                   case '-':
-                     return '+';
-                   case '_':
-                     return '/';
-                   default:
-                     return c;
-                   }
-                 });
-  if (token68str.size() & 0x3) {
-    token68str.append(4 - (token68str.size() & 0x3), '=');
+StringRef to_base64(BlockAllocator &balloc, const StringRef &token68str) {
+  // At most 3 padding '='
+  auto len = token68str.size() + 3;
+  auto iov = make_byte_ref(balloc, len + 1);
+  auto p = iov.base;
+
+  p = std::transform(std::begin(token68str), std::end(token68str), p,
+                     [](char c) {
+                       switch (c) {
+                       case '-':
+                         return '+';
+                       case '_':
+                         return '/';
+                       default:
+                         return c;
+                       }
+                     });
+
+  auto rem = token68str.size() & 0x3;
+  if (rem) {
+    p = std::fill_n(p, 4 - rem, '=');
   }
-  return;
+
+  *p = '\0';
+
+  return StringRef{iov.base, p};
 }
 
 namespace {
@@ -438,7 +473,7 @@ int levenshtein(const char *a, int alen, const char *b, int blen, int swapcost,
 }
 } // namespace
 
-void show_candidates(const char *unkopt, option *options) {
+void show_candidates(const char *unkopt, const option *options) {
   for (; *unkopt == '-'; ++unkopt)
     ;
   if (*unkopt == '\0') {
@@ -520,20 +555,16 @@ bool fieldeq(const char *uri1, const http_parser_url &u1, const char *uri2,
 
 bool fieldeq(const char *uri, const http_parser_url &u,
              http_parser_url_fields field, const char *t) {
+  return fieldeq(uri, u, field, StringRef{t});
+}
+
+bool fieldeq(const char *uri, const http_parser_url &u,
+             http_parser_url_fields field, const StringRef &t) {
   if (!has_uri_field(u, field)) {
-    if (!t[0]) {
-      return true;
-    } else {
-      return false;
-    }
-  } else if (!t[0]) {
-    return false;
+    return t.empty();
   }
-  int i, len = u.field_data[field].len;
-  const char *p = uri + u.field_data[field].off;
-  for (i = 0; i < len && t[i] && p[i] == t[i]; ++i)
-    ;
-  return i == len && !t[i];
+  auto &f = u.field_data[field];
+  return StringRef{uri + f.off, f.len} == t;
 }
 
 StringRef get_uri_field(const char *uri, const http_parser_url &u,
@@ -632,58 +663,15 @@ std::string to_numeric_addr(const Address *addr) {
   return s;
 }
 
-static int STDERR_COPY = -1;
-static int STDOUT_COPY = -1;
-
-void store_original_fds() {
-  // consider dup'ing stdout too
-  STDERR_COPY = dup(STDERR_FILENO);
-  STDOUT_COPY = STDOUT_FILENO;
-  // no race here, since it is called early
-  make_socket_closeonexec(STDERR_COPY);
-}
-
-void restore_original_fds() { dup2(STDERR_COPY, STDERR_FILENO); }
-
-void close_log_file(int &fd) {
-  if (fd != STDERR_COPY && fd != STDOUT_COPY && fd != -1) {
-    close(fd);
+void set_port(Address &addr, uint16_t port) {
+  switch (addr.su.storage.ss_family) {
+  case AF_INET:
+    addr.su.in.sin_port = htons(port);
+    break;
+  case AF_INET6:
+    addr.su.in6.sin6_port = htons(port);
+    break;
   }
-  fd = -1;
-}
-
-int open_log_file(const char *path) {
-
-  if (strcmp(path, "/dev/stdout") == 0 ||
-      strcmp(path, "/proc/self/fd/1") == 0) {
-    return STDOUT_COPY;
-  }
-
-  if (strcmp(path, "/dev/stderr") == 0 ||
-      strcmp(path, "/proc/self/fd/2") == 0) {
-    return STDERR_COPY;
-  }
-#if defined O_CLOEXEC
-
-  auto fd = open(path, O_WRONLY | O_APPEND | O_CREAT | O_CLOEXEC,
-                 S_IRUSR | S_IWUSR | S_IRGRP);
-#else // !O_CLOEXEC
-
-  auto fd =
-      open(path, O_WRONLY | O_APPEND | O_CREAT, S_IRUSR | S_IWUSR | S_IRGRP);
-
-  // We get race condition if execve is called at the same time.
-  if (fd != -1) {
-    make_socket_closeonexec(fd);
-  }
-
-#endif // !O_CLOEXEC
-
-  if (fd == -1) {
-    return -1;
-  }
-
-  return fd;
 }
 
 std::string ascii_dump(const uint8_t *data, size_t len) {
@@ -1119,29 +1107,30 @@ std::string dtos(double n) {
   return utos(static_cast<int64_t>(n)) + "." + (f.size() == 1 ? "0" : "") + f;
 }
 
-std::string make_http_hostport(const StringRef &host, uint16_t port) {
+StringRef make_http_hostport(BlockAllocator &balloc, const StringRef &host,
+                             uint16_t port) {
   if (port != 80 && port != 443) {
-    return make_hostport(host, port);
+    return make_hostport(balloc, host, port);
   }
 
   auto ipv6 = ipv6_numeric_addr(host.c_str());
 
-  std::string hostport;
-  hostport.resize(host.size() + (ipv6 ? 2 : 0));
-
-  auto p = &hostport[0];
+  auto iov = make_byte_ref(balloc, host.size() + (ipv6 ? 2 : 0) + 1);
+  auto p = iov.base;
 
   if (ipv6) {
     *p++ = '[';
   }
 
-  p = std::copy_n(host.c_str(), host.size(), p);
+  p = std::copy(std::begin(host), std::end(host), p);
 
   if (ipv6) {
     *p++ = ']';
   }
 
-  return hostport;
+  *p = '\0';
+
+  return StringRef{iov.base, p};
 }
 
 std::string make_hostport(const StringRef &host, uint16_t port) {
@@ -1169,6 +1158,34 @@ std::string make_hostport(const StringRef &host, uint16_t port) {
   return hostport;
 }
 
+StringRef make_hostport(BlockAllocator &balloc, const StringRef &host,
+                        uint16_t port) {
+  auto ipv6 = ipv6_numeric_addr(host.c_str());
+  auto serv = utos(port);
+
+  auto iov =
+      make_byte_ref(balloc, host.size() + (ipv6 ? 2 : 0) + 1 + serv.size());
+  auto p = iov.base;
+
+  if (ipv6) {
+    *p++ = '[';
+  }
+
+  p = std::copy(std::begin(host), std::end(host), p);
+
+  if (ipv6) {
+    *p++ = ']';
+  }
+
+  *p++ = ':';
+
+  p = std::copy(std::begin(serv), std::end(serv), p);
+
+  *p = '\0';
+
+  return StringRef{iov.base, p};
+}
+
 namespace {
 void hexdump8(FILE *out, const uint8_t *first, const uint8_t *last) {
   auto stop = std::min(first + 8, last);
@@ -1351,7 +1368,7 @@ uint32_t hash32(const StringRef &s) {
   return h;
 }
 
-#if !OPENSSL_101_API
+#if !OPENSSL_1_1_API
 namespace {
 EVP_MD_CTX *EVP_MD_CTX_new(void) { return EVP_MD_CTX_create(); }
 } // namespace
@@ -1359,7 +1376,7 @@ EVP_MD_CTX *EVP_MD_CTX_new(void) { return EVP_MD_CTX_create(); }
 namespace {
 void EVP_MD_CTX_free(EVP_MD_CTX *ctx) { EVP_MD_CTX_destroy(ctx); }
 } // namespace
-#endif
+#endif // !OPENSSL_1_1_API
 
 int sha256(uint8_t *res, const StringRef &s) {
   int rv;
@@ -1391,6 +1408,55 @@ int sha256(uint8_t *res, const StringRef &s) {
   return 0;
 }
 
+bool is_hex_string(const StringRef &s) {
+  if (s.size() % 2) {
+    return false;
+  }
+
+  for (auto c : s) {
+    if (!is_hex_digit(c)) {
+      return false;
+    }
+  }
+
+  return true;
+}
+
+StringRef decode_hex(BlockAllocator &balloc, const StringRef &s) {
+  auto iov = make_byte_ref(balloc, s.size() + 1);
+  auto p = iov.base;
+  for (auto it = std::begin(s); it != std::end(s); it += 2) {
+    *p++ = (hex_to_uint(*it) << 4) | hex_to_uint(*(it + 1));
+  }
+  *p = '\0';
+  return StringRef{iov.base, p};
+}
+
+StringRef extract_host(const StringRef &hostport) {
+  if (hostport[0] == '[') {
+    // assume this is IPv6 numeric address
+    auto p = std::find(std::begin(hostport), std::end(hostport), ']');
+    if (p == std::end(hostport)) {
+      return StringRef{};
+    }
+    if (p + 1 < std::end(hostport) && *(p + 1) != ':') {
+      return StringRef{};
+    }
+    return StringRef{std::begin(hostport), p + 1};
+  }
+
+  auto p = std::find(std::begin(hostport), std::end(hostport), ':');
+  if (p == std::begin(hostport)) {
+    return StringRef{};
+  }
+  return StringRef{std::begin(hostport), p};
+}
+
+std::mt19937 make_mt19937() {
+  std::random_device rd;
+  return std::mt19937(rd());
+}
+
 } // namespace util
 
 } // namespace nghttp2
